<html>
<head>
    <link rel="stylesheet" type="text/css" href="style.css">
    <meta charset="UTF-8">
    <script src="http://ajax.googleapis.com/ajax/libs/jquery/1.11.3/jquery.min.js"></script>
    <script src="rdfconv.js"></script>
</head>
<div style="float: right;">
    Updated {{ date }}
</div>
{% for node in nodes %}
    {% include 'node.html' with node=node %}
{% endfor %}
</html>