<div class="rdf_obj" id="{{ node.node_id }}">
    <div class="title">
        <h1>{{ node.title }}</h1>
    </div>
    <div class="type">
        <h2>{{ node.rdf_type }}</h2>
    </div>
    <div class="desc">{{ node.desc | safe }} </div>
    <div class="full_info">
        <table>
            <tbody>
            {% for attr in node.attributes %}
                <tr>
                    <td><a href="{{ attr.pred_link }}">{{ attr.pred_title }}</a></td>
                    <td>
                    {% for obj in attr.objs %}
                        {% if obj.link %}
                            <a href="{{ obj.link }}">{{ obj.title }}</a><br />
                        {% else %}
                            {{ obj.title | safe }}
                        {% endif %}
                    {% endfor %}
                    </td>
                </tr>
            {% endfor %}
            </tbody>
        </table>
    </div>
    <a class="show_more">Show more</a>
</div>